//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * インフォメーションレンダラヘッダ
 * @author Junpee
 */

#ifndef INFORMATION_RENDERER_H_
#define INFORMATION_RENDERER_H_

#include <Core/Container/ArrayList.h>

namespace Lamp{

class PrimitiveRenderer;
class Scene;
class Camera;
class Mesh;

//------------------------------------------------------------------------------
/**
 * インフォメーションレンダラ
 */
class InformationRenderer{
public:
	//--------------------------------------------------------------------------
	// 生成、破棄
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 */
	InformationRenderer();

	/**
	 * デストラクタ
	 */
	virtual ~InformationRenderer();

	//--------------------------------------------------------------------------
	// レンダリング
	//--------------------------------------------------------------------------
	/**
	 * レンダリング準備を行う
	 * @param scene レンダリングを行うシーン
	 */
	virtual void renderingSetup(Scene* scene);

	/**
	 * レンダリングを行う
	 */
	virtual void rendering();

	//--------------------------------------------------------------------------
	// 描画フラグ
	//--------------------------------------------------------------------------
	/**
	 * グリッドの描画フラグ設定
	 * @param isDrawnGrid グリッドを描画するならtrue
	 */
	virtual void setDrawnGrid(bool isDrawnGrid){ isDrawnGrid_ = isDrawnGrid; }

	/**
	 * グリッドを描画するか
	 * @return グリッドを描画するならtrue
	 */
	virtual bool isDrawnGrid() const{ return isDrawnGrid_; }

	//--------------------------------------------------------------------------
	/**
	 * 軸の描画フラグ設定
	 * @param isDrawnAxis 軸を描画するならtrue
	 */
	virtual void setDrawnAxis(bool isDrawnAxis){ isDrawnAxis_ = isDrawnAxis; }

	/**
	 * 軸を描画するか
	 * @return 軸を描画するならtrue
	 */
	virtual bool isDrawnAxis() const{ return isDrawnAxis_; }

	//--------------------------------------------------------------------------
	/**
	 * メッシュバウンディングの描画フラグ設定
	 * @param isDrawnMeshBounding メッシュバウンディングを描画するならtrue
	 */
	virtual void setDrawnMeshBounding(bool isDrawnMeshBounding){
		isDrawnMeshBounding_ = isDrawnMeshBounding;
	}

	/**
	 * メッシュバウンディングを描画するか
	 * @return メッシュバウンディングを描画するならtrue
	 */
	virtual bool isDrawnMeshBounding() const{ return isDrawnMeshBounding_; }

	//--------------------------------------------------------------------------
	/**
	 * ボーンの描画フラグ設定
	 * @param isDrawnBone ボーンを描画するならtrue
	 */
	virtual void setDrawnBone(bool isDrawnBone){ isDrawnBone_ = isDrawnBone; }

	/**
	 * ボーンを描画するか
	 * @return ボーンを描画するならtrue
	 */
	virtual bool isDrawnBone() const{ return isDrawnBone_; }

protected:
	//--------------------------------------------------------------------------
	/**
	 * グリッドのセットアップ
	 */
	virtual void setupGrid();

	/**
	 * 軸のセットアップ
	 */
	virtual void setupAxis();

	/**
	 * メッシュバウンディングのセットアップ
	 */
	virtual void setupMeshBounding();

	/**
	 * ボーンのセットアップ
	 */
	virtual void setupBone();

	//--------------------------------------------------------------------------
	/// プリミティブレンダラ
	PrimitiveRenderer* renderer_;
	/// シーン
	Scene* scene_;
	/// カメラ
	Camera* camera_;
	/// メッシュリスト
	ArrayList<Mesh*> meshList_;

	/// グリッドを描画するか
	bool isDrawnGrid_;
	/// 軸を描画するか
	bool isDrawnAxis_;
	/// メッシュバウンディングを描画するか
	bool isDrawnMeshBounding_;
	/// ボーンを描画するか
	bool isDrawnBone_;

	/// メッシュバウンディングカラー
	static const Color4c meshBoundingColor_;
	/// ボーンカラー
	static const Color4c boneColor_;

private:
	//--------------------------------------------------------------------------
	// コピーコンストラクタの隠蔽
	InformationRenderer(const InformationRenderer& copy);

	// 代入コピーの隠蔽
	void operator =(const InformationRenderer& copy);

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of INFORMATION_RENDERER_H_
//------------------------------------------------------------------------------
